﻿Public Class Derived

#Region "Declarations"
    Public Shared Vb As Double
    Public Shared BL As Double
    Public Shared Cms, Cmp, Cmp_m As Double
    Public Shared Mms As Double
    Public Shared Vasp As Double
    Public Shared Rms As Double
    Public Shared Fsp_m, Fb_m, Vab_m, Qmp_m As Double
    Public Shared Eff As Double
    Public Shared SPL1W
    Public Shared SPL283V
    Public Shared Vc As Double  'Applied voltage
    Public Shared alignment As Double
    Public Shared Max_filter As Double
    Public Shared Max_output As Double
    Public Shared Voltage_in As Double
    Const Vent_End_correction As Double = 0.732
    Public Shared speaker As New Woofer_spec
    Public Shared enclosure As New Box_spec
    Public Shared Fs, Le, Re, Rs, Qes, Qms, Qts, Sd, Vas, Xmax, Wd, Fb As Double  'Local versions of the woofer data

#End Region

#Region "Window load and close"

    Private Sub Derived_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'This gets executed when the window loads
        'Load the default PR values
        Color_Chooser.Color_set(Me)
        Call Update_PR_values()
        Call Calc_derived()
    End Sub

    Private Sub derived_done_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles derived_done_Button.Click
        Box_model.Derived_param_chart_active = False
        Box_model.DerivedParametersToolStripMenuItem.Checked = False
        Me.Close()
    End Sub

#End Region

#Region "Calculations"

    Public Sub Calc_derived()

        Dim Pe As Double    'Local version of other project data

        'copy state into local variables
        Select Case Box_model.Selected_Driver
            Case Box_model.Selected_Woofer_box.woofer
                speaker = Main.BCD.Spkr.driver1.TS_Parameters
                enclosure = Main.BCD.Spkr.driver1.Enclosure
            Case Box_model.Selected_Woofer_box.mid1
                speaker = Main.BCD.Spkr.driver2.TS_Parameters
                enclosure = Main.BCD.Spkr.driver2.Enclosure
            Case Box_model.Selected_Woofer_box.what_if
                speaker = Main.BCD.Cabinet.Woofer
                enclosure = Main.BCD.Cabinet.Box
        End Select

        'Copy the project data into local variables to make the equations more readable
        Fs = speaker.Fs
        Le = speaker.Le
        Re = speaker.Re * speaker.Config.ReX
        Sd = speaker.Sd / 10000 * speaker.Config.SdX
        Vas = speaker.Vas / 1000 * speaker.Config.VasX
        Qes = speaker.Qes
        Qms = speaker.Qms
        Qts = speaker.Qts
        Rs = speaker.Rs
        Xmax = speaker.Xmax

        Pe = Main.BCD.Amp.Woofer_amp.Watts
        Main.BCD.Amp.Woofer_amp.Impedance = Re
        Wd = 2 * Math.Sqrt(Sd / Math.PI) * 1000

        If enclosure.Box_Type = Box_model.sealed_box Then
            Fb = 0.000001
        Else
            Fb = enclosure.Target_Fb
        End If

        Call Update_Vasp(Vas)      'create Vasp
        Call DCR_calc.Series_res_TextChanged(Nothing, Nothing)

        'Call Amplifier.Amp_calc()
        Call Update_Vb()
        Call Update_Cms(Sd)
        Call Update_Mms(Fs)
        Call Update_BL(Fs, Re, Qes)
        Call Update_Rms(Fs, Qms)
        Call Update_Efficiency(Fs, Qes)
        Call Update_Vc(Re, Qes)
        DCR_TextBox.Text = Str(Math.Round(Box_model.DCR, 3))     'just fill in the text box
        Qts_TextBox.Text = Str(Math.Round(Qts, 3))
        Call Update_SPL1W(Re, Rs)
        Call Update_SPL283V(Re, Rs)
        Call Calc_Rec_vol(Fs, Qts)                   'calculate recommended volume for QTC
        Call Update_alignment(Qts, Vas, Fb, Fs)
        Call Update_Port_diameter(Xmax, Fb, Wd)
        Call Update_Max_output(Re)
        Call Update_Lv(Fb)
        Call calc_Cmp_and_Vasp()
        Call Update_added_mass()
        Call Update_Benson()

    End Sub

    Sub Update_Vb()
        Select Case enclosure.Box_Type
            Case Box_model.sealed_box
                Vb = enclosure.Volume_sealed
            Case Box_model.vented_box
                Vb = enclosure.Volume_vented
            Case Box_model.PR_box
                Vb = enclosure.Volume_PR
        End Select
    End Sub

    Sub Update_Vasp(ByVal Vas)
        'Vasp = Vas * speaker.Config.VasX
        'Vas_total = Vas / 1000 * speaker.Config.VasX
        Vasp = Vas * 1000
    End Sub

    Sub Update_Cms(ByVal Sd)
        'Vas/(Rho0*Cair^2*Sd^2)
        Cms = Vas / (Box_model.Rho0 * (Box_model.Cair ^ 2) * (Sd ^ 2))
        Cms_TextBox.Text = Str(Math.Round(1000 * Cms, 3))
    End Sub
    Sub Update_Mms(ByVal Fs)
        '1/((2*PI()*Fs)^2*Cms)
        Mms = 1 / (Math.Pow(2 * Math.PI * Fs, 2) * Cms)
        Mms_TextBox.Text = Str(Math.Round(Mms, 3))
    End Sub

    Sub Update_BL(ByVal Fs, ByVal Re, ByVal Qes)
        '((2*PI()*Fs*Re*Mms)/Qes)^0.5 
        BL = Math.Sqrt((2 * Math.PI * Fs * Re * Mms) / Qes)
        BL_TextBox.Text = Str(Math.Round(BL, 3))
    End Sub

    Sub Update_Rms(ByVal Fs, ByVal Qms)
        '1/(2*PI()*Fs*Cms*Qms)
        Rms = 1 / (2 * Math.PI * Fs * Cms * Qms)
        Rms_TextBox.Text = Str(Math.Round(Rms, 3))
    End Sub

    Sub Update_Efficiency(ByVal Fs, ByVal Qes)
        '(0.0000000009614*Fs^3*Vasp)/Qes
        Eff = (0.00000009614 * (Fs ^ 3) * Vasp) / Qes
        Eff_TextBox.Text = Str(Math.Round(Eff, 3))
    End Sub

    Sub Update_Vc(ByVal Re, ByVal Qes)
        '(Parameters!P30*Re)^0.5    P30 = 10^(P31/10)    P31 = 10*LOG(Avail_power)-'SPL Calc'!R3      'SPL Calc'!R3=Max Gain from active filters
        'for now, just assume max Gain is 0db
        Vc = (Main.BCD.Amp.Woofer_amp.Watts * Re) ^ 0.5
        AP_TextBox.Text = Str(Math.Round(Vc, 3))
    End Sub

    Sub Update_SPL1W(ByVal Re, ByVal Rs)
        '112.1+10*LOG(C43)+10*LOG(Re/(Re+C20))
        SPL1W = 112.1 + 10 * Math.Log10(Eff / 100) + 10 * Math.Log10(Re / (Re + Rs))
        SPLW_TextBox.Text = Str(Math.Round(SPL1W, 3))
    End Sub

    Sub Update_SPL283V(ByVal Re, ByVal Rs)
        'C39+20*LOG(SQRT(8/(Re+C20)))
        SPL283V = SPL1W + 20 * Math.Log10(Math.Sqrt(8 / (Re + Rs)))
        SplV_TextBox.Text = Str(Math.Round(SPL283V, 3))
    End Sub

    'Calculates alignment, h and alpha -- just for textbox display.  Not used for calculations--that is done in Benson module
    Sub Update_alignment(ByVal Qts, ByVal Vas, ByVal Fb, ByVal Fs)
        alignment = enclosure.Volume_vented / (Vasp * Qts ^ 2)
        Align_textbox.Text = Str(Math.Round(alignment, 3))
        h_textbox.Text = Str(Math.Round(Benson.h, 3))
        alpha_textbox.Text = Str(Math.Round(Benson.alpha, 3))
    End Sub

    Sub Update_Port_diameter(ByVal Xmax, ByVal Fb, ByVal Wd)
        Box_model.P_diam_min = 39.37 * (Math.Sqrt(Fb * ((Xmax / 1000) * (Math.PI * ((0.5 * (Wd / 1000)) ^ 2)))))
        Port_min_textbox.Text = Str(Math.Round(Box_model.P_diam_min, 3))
    End Sub

    Sub Calc_Rec_vol(ByVal Fs, ByVal Qts)
        Dim VBv1 As Double
        Dim SBv1 As Double
        VBv1 = 20 * Vasp * Qts ^ 3.3
        Box_model.Calc_Vented_vol = VBv1
        Box_model.Calc_Vented_Fb = (Fs / Qts) * 0.39
        SBv1 = Vasp / (((1 / Qts) / (1 / Box_model.Qtc1)) ^ 2 - 1)
        Box_model.Calc_Sealed_vol = SBv1
        Box_model.Calc_PR_Vol = Box_model.Calc_Vented_vol
    End Sub

    Sub Update_Max_output(ByVal Re)

        'temporary values until amplifier routines are done
        Dim temp As Double
        Max_filter = -100
        For n = 0 To Main.Samples - 1
            temp = 10 * Math.Log10(Mag(Main.Arrays.Amp_EQ_response.EQ(n)))
            If temp > Max_filter Then Max_filter = temp
        Next n

        Box_model.P_eff = Main.BCD.Amp.Woofer_amp.Watts

        For n = 0 To Main.Samples - 1

        Next

        Box_model.P_eff_adj = 10 * Math.Log10(Box_model.P_eff) - Box_model.Max_Gain
        Max_output = SPL1W + Box_model.P_eff_adj
        Max_Out_TextBox.Text = Str(Math.Round(Max_output, 3))
        Voltage_in = Math.Sqrt(10 ^ (Box_model.P_eff_adj / 10) * Re) * Math.Sqrt(2)
        'Voltage_in_TextBox.Text = Str(Math.Round(Voltage_in, 3))
    End Sub

    Sub Update_Lv(ByVal Fb)
        Dim Lv As Double    'replace with XML vent.Lv
        '=2350*L33^2/(I34^2*(I33/nV))-(L35*L33)
        '=2350*Dv^2 / (Fb^2*(Vb/Nv))-(Vent_End_correction*Dv)
        Lv = 1.5161 * enclosure.Vent.Dv ^ 2 / (Fb ^ 2 * (Vb / 1000 / enclosure.Vent.Nv)) - (Vent_End_correction * enclosure.Vent.Dv / 39.37)
        enclosure.Vent.Lv = Lv * 39.37
    End Sub

    Sub Update_Fb_from_Lv(ByVal new_vent, ByRef new_target)
        Dim Lv As Double
        Dim port_correction_factor As Double
        Dim Vb_adj As Double

        Lv = new_vent / 39.37
        port_correction_factor = Vent_End_correction * enclosure.Vent.Dv / 39.37

        Lv = Lv + port_correction_factor
        Vb_adj = Vb / 1000 / enclosure.Vent.Nv

        'Calculate the Fb based on the entered vent length
        enclosure.Target_Fb = Math.Sqrt((1.5161 * enclosure.Vent.Dv ^ 2) / (Lv * Vb_adj))
        new_target = enclosure.Target_Fb
    End Sub

    Sub calc_Cmp_and_Vasp()

        'Dim temp As Double
        'temp = Main.Rho0 * Main.Cair ^ 2 * (Main.BCD.Cabinet.box.PR.Sd / 1000) ^ 2
        'Check whether Vas for the PR is within 5% of calculated value.  If not, replace it with calculated value
        'If Math.Abs(Main.BCD.Cabinet.box.PR.Vas - temp) > Main.BCD.Cabinet.box.PR.Vas * 0.05 Then
        'Main.BCD.Cabinet.box.PR.Vas = temp
        'End If
        'Calculated_Vasp_textbox.Text = Str(Math.Round(temp, 3))

        'If Cms not provided, then derive it from Vas and Sd
        If enclosure.PR.Cms = 0 Then
            enclosure.PR.Cms = enclosure.PR.Vas / (1000 * Box_model.Rho0 * Box_model.Cair ^ 2 * (enclosure.PR.Sd / 10000 * enclosure.PR.Np) ^ 2) * 1000
            'enclosure.PR.Cms = 1000 / ((Math.PI * 2 * enclosure.Target_Fb) ^ 2 * enclosure.PR.Mms)
        End If
        'update the derived parameters form
        Cmp_calculated_textbox.Text = Str(Math.Round(enclosure.PR.Cms, 5))

        Qmp_m = enclosure.PR.Qms * Math.Sqrt((enclosure.PR.Mms + enclosure.PR.Mass_added) / enclosure.PR.Mms)
        Qmp_withmass_TextBox.Text = Str(Math.Round(Qmp_m, 3))

        Vab_m = enclosure.Volume_PR * (1 + 0.9 / enclosure.Qa ^ 0.9)
        Vab_withabsorbtion_TextBox.Text = Str(Math.Round(Vab_m, 3))

        'Cmp = enclosure.PR.Vas / (1000 * Main.Rho0 * Main.Cair ^ 2 * enclosure.PR.Sd ^ 2)
        Cmp_m = (enclosure.PR.Vas * Vab_m / (enclosure.PR.Vas + Vab_m)) / (1000 * Box_model.Rho0 * Box_model.Cair ^ 2 * (enclosure.PR.Sd / 10000) ^ 2)
        Cmp_mass_added_TextBox.Text = Str(Math.Round(Cmp_m, 5))

    End Sub

    Sub Update_added_mass()
        'Adjust variables for added mass
        Fsp_m = enclosure.PR.Fs * Math.Sqrt(enclosure.PR.Mms / (enclosure.PR.Mms + enclosure.PR.Mass_added))
        Fsp_withmass_TextBox.Text = Str(Math.Round(Fsp_m, 3))

        Fb_m = Benson.Fbtp
        Fb_withmass_TextBox.Text = Str(Math.Round(Fb_m, 3))

        'mass to add = =1/((PI()*2*Benson!BB33)^2*BB43)-1/((PI()*2*Benson!BB17)^2*BB41) 
        Box_model.Mass_to_add = 1 / ((Math.PI * 2 * enclosure.Target_Fb) ^ 2 * Cmp_m) - 1 / ((Math.PI * 2 * enclosure.PR.Fs) ^ 2 * enclosure.PR.Cms / 1000)

        'now convert to grams and display the result
        Box_model.Mass_to_add = Box_model.Mass_to_add * 1000 / enclosure.PR.Np
    End Sub

    Public Sub Update_PR_values()
        Fsp_textbox.Text = Str(enclosure.PR.Fs)
        Vasp_textbox.Text = Str(enclosure.PR.Vas)
        Mmp_textbox.Text = Str(enclosure.PR.Mms)
        Qmp_textbox.Text = Str(enclosure.PR.Qms)
        Cmp_mass_added_TextBox.Text = Str(enclosure.PR.Cms)
        Rmp_textbox.Text = Str(enclosure.PR.Rms)

    End Sub

#End Region

#Region "Controls (Benson Parameters Button)"
    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        Call Benson.Benson_init()       'this gets moved
        Call Benson.Benson_Coef_calc()
        Dim Benson_window As New Benson
        Benson_window.Show()
        Call Benson.Benson_SPL_calc()

    End Sub
#End Region

#Region "Calls to other modules"

    Public Sub Update_Benson()
        Call Benson.Benson_init()
        Call Benson.Benson_Coef_calc()          'calculate the coefficients for the Benson model
        Call Benson.Benson_SPL_calc()               'run the model

        If Main.Woofer_box_window_open_flag = True Then
            'This suppresses updates of the charts if the window is closed
            Call Main.Update_WooferBox_Charts_delegate()
        End If
    End Sub
#End Region

End Class